@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -40,9 +40,7 @@
 #include <stdlib.h>
 #include <stdbool.h>
 
-#ifdef HAVE_JSON
 #include <json.h>
-#endif
 
 #include "clamav.h"
 #include "dconf.h"
@@ -60,9 +58,7 @@
 #include "yara_clam.h"
 #endif
 
-#if HAVE_LIBXML2
 #define CLAMAV_MIN_XMLREADER_FLAGS (XML_PARSE_NOERROR | XML_PARSE_NONET)
-#endif
 
 /*
  * CL_FLEVEL is the signature f-level specific to the current code and
@@ -73,7 +69,7 @@
  * in re-enabling affected modules.
  */
 
-#define CL_FLEVEL 168
+#define CL_FLEVEL 212
 #define CL_FLEVEL_DCONF CL_FLEVEL
 #define CL_FLEVEL_SIGTOOL CL_FLEVEL
 
@@ -191,6 +187,7 @@ typedef struct recursion_level_tag {
 } recursion_level_t;
 
 typedef void *evidence_t;
+typedef void *onedump_t;
 
 /* internal clamav context */
 typedef struct cli_ctx_tag {
@@ -217,10 +214,8 @@ typedef struct cli_ctx_tag {
 #ifdef HAVE__INTERNAL__SHA_COLLECT
     int sha_collect;
 #endif
-#ifdef HAVE_JSON
     struct json_object *properties;
     struct json_object *wrkproperty;
-#endif
     struct timeval time_limit;
     bool limit_exceeded; /* To guard against alerting on limits exceeded more than once, or storing that in the JSON metadata more than once. */
     bool abort_scan;     /* So we can guarantee a scan is aborted, even if CL_ETIMEOUT/etc. status is lost in the scan recursion stack. */
@@ -327,6 +322,7 @@ struct cl_engine {
     char *tmpdir;
     uint32_t keeptmp;
     uint64_t engine_options;
+    uint32_t cache_size;
 
     /* Limits */
     uint32_t maxscantime;         /* Time limit (in milliseconds) */
@@ -416,6 +412,7 @@ struct cl_engine {
     void *cb_sigload_ctx;
     clcb_hash cb_hash;
     clcb_meta cb_meta;
+    clcb_generic_data cb_vba;
     clcb_file_props cb_file_props;
     clcb_progress cb_sigload_progress;
     void *cb_sigload_progress_ctx;
@@ -491,6 +488,7 @@ struct cl_settings {
     enum bytecode_mode bytecode_mode;
     char *pua_cats;
     uint64_t engine_options;
+    uint32_t cache_size;
 
     /* callbacks */
     clcb_pre_cache cb_pre_cache;
@@ -565,6 +563,9 @@ extern LIBCLAMAV_EXPORT int have_rar;
 #define SCAN_PARSE_OLE2 (ctx->options->parse & CL_SCAN_PARSE_OLE2)
 #define SCAN_PARSE_HTML (ctx->options->parse & CL_SCAN_PARSE_HTML)
 #define SCAN_PARSE_PE (ctx->options->parse & CL_SCAN_PARSE_PE)
+#define SCAN_PARSE_ONENOTE (ctx->options->parse & CL_SCAN_PARSE_ONENOTE)
+#define SCAN_PARSE_IMAGE (ctx->options->parse & CL_SCAN_PARSE_IMAGE)
+#define SCAN_PARSE_IMAGE_FUZZY_HASH (ctx->options->parse & CL_SCAN_PARSE_IMAGE_FUZZY_HASH)
 
 #define SCAN_HEURISTIC_BROKEN (ctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN)
 #define SCAN_HEURISTIC_BROKEN_MEDIA (ctx->options->heuristic & CL_SCAN_HEURISTIC_BROKEN_MEDIA)
@@ -931,13 +932,34 @@ static inline int cli_getpagesize(void)
 #endif /* HAVE_SYSCONF_SC_PAGESIZE */
 #endif /* _WIN32 */
 
-void *cli_malloc(size_t nmemb);
-void *cli_calloc(size_t nmemb, size_t size);
+/**
+ * @brief Wrapper around malloc that limits how much may be allocated to CLI_MAX_ALLOCATION.
+ *
+ * Please use CLI_MAX_MALLOC_OR_GOTO_DONE() with `goto done;` error handling instead.
+ *
+ * @param ptr
+ * @param size
+ * @return void*
+ */
+void *cli_max_malloc(size_t nmemb);
+
+/**
+ * @brief Wrapper around calloc that limits how much may be allocated to CLI_MAX_ALLOCATION.
+ *
+ * Please use CLI_MAX_CALLOC_OR_GOTO_DONE() with `goto done;` error handling instead.
+ *
+ * @param ptr
+ * @param size
+ * @return void*
+ */
+void *cli_max_calloc(size_t nmemb, size_t size);
 
 /**
  * @brief Wrapper around realloc that limits how much may be allocated to CLI_MAX_ALLOCATION.
  *
- * Please use CLI_REALLOC() with `goto done;` error handling instead.
+ * Please use CLI_MAX_REALLOC_OR_GOTO_DONE() with `goto done;` error handling instead.
+ *
+ * NOTE: cli_max_realloc() will NOT free ptr if size==0. It is safe to free ptr after `done:`.
  *
  * IMPORTANT: This differs from realloc() in that if size==0, it will NOT free the ptr.
  *
@@ -945,25 +967,63 @@ void *cli_calloc(size_t nmemb, size_t size);
  * @param size
  * @return void*
  */
-void *cli_realloc(void *ptr, size_t size);
+void *cli_max_realloc(void *ptr, size_t size);
 
 /**
  * @brief Wrapper around realloc that limits how much may be allocated to CLI_MAX_ALLOCATION.
  *
- * Please use CLI_REALLOC() with `goto done;` error handling instead.
+ * Please use CLI_MAX_REALLOC_OR_GOTO_DONE() with `goto done;` error handling instead.
  *
  * IMPORTANT: This differs from realloc() in that if size==0, it will NOT free the ptr.
  *
- * WARNING: This differs from cli_realloc() in that it will free the ptr if the allocation fails.
+ * WARNING: This differs from cli_max_realloc() in that it will free the ptr if the allocation fails.
  * If you're using `goto done;` error handling, this may result in a double-free!!
  *
  * @param ptr
  * @param size
  * @return void*
  */
-void *cli_realloc2(void *ptr, size_t size);
+void *cli_max_realloc_or_free(void *ptr, size_t size);
+
+/**
+ * @brief Wrapper around realloc that, unlike some variants of realloc, will not free the ptr if size==0.
+ *
+ * Please use CLI_MAX_REALLOC_OR_GOTO_DONE() with `goto done;` error handling instead.
+ *
+ * IMPORTANT: This differs from realloc() in that if size==0, it will NOT free the ptr.
+ *
+ * @param ptr
+ * @param size
+ * @return void*
+ */
+void *cli_safer_realloc(void *ptr, size_t size);
+
+/**
+ * @brief Wrapper around realloc that, unlike some variants of realloc, will not free the ptr if size==0.
+ *
+ * Please use CLI_SAFER_REALLOC_OR_GOTO_DONE() with `goto done;` error handling instead.
+ *
+ * IMPORTANT: This differs from realloc() in that if size==0, it will NOT free the ptr.
+ *
+ * WARNING: This differs from cli_safer_realloc() in that it will free the ptr if the allocation fails.
+ * If you're using `goto done;` error handling, this may result in a double-free!!
+ *
+ * @param ptr
+ * @param size
+ * @return void*
+ */
+void *cli_safer_realloc_or_free(void *ptr, size_t size);
+
+/**
+ * @brief Wrapper around strdup that does a NULL check.
+ *
+ * Please use CLI_STRDUP_OR_GOTO_DONE() with `goto done;` error handling instead.
+ *
+ * @param s
+ * @return char* Returns the allocated string or NULL if allocation failed. This includes if allocation fails because s==NULL.
+ */
+char *cli_safer_strdup(const char *s);
 
-char *cli_strdup(const char *s);
 int cli_rmdirs(const char *dirname);
 char *cli_hashstream(FILE *fs, unsigned char *digcpy, int type);
 char *cli_hashfile(const char *filename, int type);
@@ -984,7 +1044,7 @@ const char *cli_gettmpdir(void);
  * @brief Sanitize a relative path, so it cannot have a negative depth.
  *
  * Caller is responsible for freeing the sanitized filepath.
- * The optioal sanitized_filebase output param is a pointer into the filepath,
+ * The optional sanitized_filebase output param is a pointer into the filepath,
  * if set, and does not need to be freed.
  *
  * @param filepath                  The filepath to sanitize
@@ -1077,7 +1137,7 @@ int cli_bitset_set(bitset_t *bs, unsigned long bit_offset);
 int cli_bitset_test(bitset_t *bs, unsigned long bit_offset);
 const char *cli_ctime(const time_t *timep, char *buf, const size_t bufsize);
 
-cl_error_t cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, unsigned long need2, unsigned long need3);
+cl_error_t cli_checklimits(const char *who, cli_ctx *ctx, uint64_t need1, uint64_t need2, uint64_t need3);
 
 /**
  * @brief Call before scanning a file to determine if we should scan it, skip it, or abort the entire scanning process.
@@ -1091,7 +1151,6 @@ cl_error_t cli_checklimits(const char *who, cli_ctx *ctx, unsigned long need1, u
  */
 cl_error_t cli_updatelimits(cli_ctx *ctx, size_t needed);
 
-unsigned long cli_getsizelimit(cli_ctx *, unsigned long);
 int cli_matchregex(const char *str, const char *regex);
 void cli_qsort(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *));
 void cli_qsort_r(void *a, size_t n, size_t es, int (*cmp)(const void *, const void *, const void *), void *arg);
@@ -1236,103 +1295,150 @@ uint8_t cli_get_debug_flag(void);
  */
 uint8_t cli_set_debug_flag(uint8_t debug_flag);
 
-#ifndef STRDUP
-#define STRDUP(buf, var, ...) \
-    do {                      \
-        var = strdup(buf);    \
-        if (NULL == var) {    \
-            do {              \
-                __VA_ARGS__;  \
-            } while (0);      \
-            goto done;        \
-        }                     \
+#ifndef CLI_SAFER_STRDUP_OR_GOTO_DONE
+/**
+ * @brief Wrapper around strdup that does a NULL check.
+ *
+ * This macro requires `goto done;` error handling.
+ *
+ * @param buf   The string to duplicate.
+ * @param var   The variable to assign the allocated string to.
+ * @param ...   The error handling code to execute if the allocation fails.
+ */
+#define CLI_SAFER_STRDUP_OR_GOTO_DONE(buf, var, ...) \
+    do {                                             \
+        var = cli_safer_strdup(buf);                 \
+        if (NULL == var) {                           \
+            do {                                     \
+                __VA_ARGS__;                         \
+            } while (0);                             \
+            goto done;                               \
+        }                                            \
     } while (0)
 #endif
 
-#ifndef CLI_STRDUP
-#define CLI_STRDUP(buf, var, ...) \
-    do {                          \
-        var = cli_strdup(buf);    \
-        if (NULL == var) {        \
-            do {                  \
-                __VA_ARGS__;      \
-            } while (0);          \
-            goto done;            \
-        }                         \
-    } while (0)
-#endif
-
-#ifndef FREE
-#define FREE(var)          \
-    do {                   \
-        if (NULL != var) { \
-            free(var);     \
-            var = NULL;    \
-        }                  \
-    } while (0)
-#endif
-
-#ifndef MALLOC
-#define MALLOC(var, size, ...) \
-    do {                       \
-        var = malloc(size);    \
-        if (NULL == var) {     \
-            do {               \
-                __VA_ARGS__;   \
-            } while (0);       \
-            goto done;         \
-        }                      \
-    } while (0)
-#endif
-
-#ifndef CLI_MALLOC
-#define CLI_MALLOC(var, size, ...) \
+#ifndef CLI_FREE_AND_SET_NULL
+/**
+ * @brief Wrapper around `free()` to ensure you reset the variable to NULL so as to prevent a double-free.
+ *
+ * @param var The variable to free and set to NULL.
+ */
+#define CLI_FREE_AND_SET_NULL(var) \
     do {                           \
-        var = cli_malloc(size);    \
-        if (NULL == var) {         \
-            do {                   \
-                __VA_ARGS__;       \
-            } while (0);           \
-            goto done;             \
+        if (NULL != var) {         \
+            free((void *)var);     \
+            var = NULL;            \
         }                          \
     } while (0)
 #endif
 
-#ifndef CALLOC
-#define CALLOC(var, nmemb, size, ...) \
-    do {                              \
-        (var) = calloc(nmemb, size);  \
-        if (NULL == var) {            \
-            do {                      \
-                __VA_ARGS__;          \
-            } while (0);              \
-            goto done;                \
-        }                             \
+#ifndef CLI_MALLOC_OR_GOTO_DONE
+/**
+ * @brief Wrapper around malloc that will `goto done;` if the allocation fails.
+ *
+ * This macro requires `goto done;` error handling.
+ *
+ * @param ptr   The variable to assign the allocated memory to.
+ * @param size  The size of the memory to allocate.
+ * @param ...   The error handling code to execute if the allocation fails.
+ */
+#define CLI_MALLOC_OR_GOTO_DONE(var, size, ...) \
+    do {                                        \
+        var = malloc(size);                     \
+        if (NULL == var) {                      \
+            do {                                \
+                __VA_ARGS__;                    \
+            } while (0);                        \
+            goto done;                          \
+        }                                       \
     } while (0)
 #endif
 
-#ifndef CLI_CALLOC
-#define CLI_CALLOC(var, nmemb, size, ...) \
-    do {                                  \
-        (var) = cli_calloc(nmemb, size);  \
-        if (NULL == var) {                \
-            do {                          \
-                __VA_ARGS__;              \
-            } while (0);                  \
-            goto done;                    \
-        }                                 \
+#ifndef CLI_MAX_MALLOC_OR_GOTO_DONE
+/**
+ * @brief Wrapper around malloc that limits how much may be allocated to CLI_MAX_ALLOCATION.
+ *
+ * This macro requires `goto done;` error handling.
+ *
+ * @param var   The variable to assign the allocated memory to.
+ * @param size  The size of the memory to allocate.
+ * @param ...   The error handling code to execute if the allocation fails.
+ */
+#define CLI_MAX_MALLOC_OR_GOTO_DONE(var, size, ...) \
+    do {                                            \
+        var = cli_max_malloc(size);                 \
+        if (NULL == var) {                          \
+            do {                                    \
+                __VA_ARGS__;                        \
+            } while (0);                            \
+            goto done;                              \
+        }                                           \
     } while (0)
 #endif
 
-#ifndef VERIFY_POINTER
-#define VERIFY_POINTER(ptr, ...) \
-    do {                         \
-        if (NULL == ptr) {       \
-            do {                 \
-                __VA_ARGS__;     \
-            } while (0);         \
-            goto done;           \
-        }                        \
+#ifndef CLI_CALLOC_OR_GOTO_DONE
+/**
+ * @brief Wrapper around calloc that will `goto done;` if the allocation fails.
+ *
+ * This macro requires `goto done;` error handling.
+ *
+ * @param var   The variable to assign the allocated memory to.
+ * @param nmemb The number of elements to allocate.
+ * @param size  The size of each element.
+ * @param ...   The error handling code to execute if the allocation fails.
+ */
+#define CLI_CALLOC_OR_GOTO_DONE(var, nmemb, size, ...) \
+    do {                                               \
+        (var) = calloc(nmemb, size);                   \
+        if (NULL == var) {                             \
+            do {                                       \
+                __VA_ARGS__;                           \
+            } while (0);                               \
+            goto done;                                 \
+        }                                              \
+    } while (0)
+#endif
+
+#ifndef CLI_MAX_CALLOC_OR_GOTO_DONE
+/**
+ * @brief Wrapper around calloc that limits how much may be allocated to CLI_MAX_ALLOCATION.
+ *
+ * This macro requires `goto done;` error handling.
+ *
+ * @param var   The variable to assign the allocated memory to.
+ * @param nmemb The number of elements to allocate.
+ * @param size  The size of each element.
+ * @param ...   The error handling code to execute if the allocation fails.
+ */
+#define CLI_MAX_CALLOC_OR_GOTO_DONE(var, nmemb, size, ...) \
+    do {                                                   \
+        (var) = cli_max_calloc(nmemb, size);               \
+        if (NULL == var) {                                 \
+            do {                                           \
+                __VA_ARGS__;                               \
+            } while (0);                                   \
+            goto done;                                     \
+        }                                                  \
+    } while (0)
+#endif
+
+#ifndef CLI_VERIFY_POINTER_OR_GOTO_DONE
+/**
+ * @brief Wrapper around a NULL-check that will `goto done;` if the pointer is NULL.
+ *
+ * This macro requires `goto done;` error handling.
+ *
+ * @param ptr   The pointer to verify.
+ * @param ...   The error handling code to execute if the pointer is NULL.
+ */
+#define CLI_VERIFY_POINTER_OR_GOTO_DONE(ptr, ...) \
+    do {                                          \
+        if (NULL == ptr) {                        \
+            do {                                  \
+                __VA_ARGS__;                      \
+            } while (0);                          \
+            goto done;                            \
+        }                                         \
     } while (0)
 #endif
 
@@ -1341,37 +1447,48 @@ uint8_t cli_set_debug_flag(uint8_t debug_flag);
  *
  * IMPORTANT: This differs from realloc() in that if size==0, it will NOT free the ptr.
  *
- * NOTE: cli_realloc() will NOT free ptr if size==0. It is safe to free ptr after `done:`.
+ * NOTE: cli_max_realloc() will NOT free ptr if size==0. It is safe to free ptr after `done:`.
  *
- * @param ptr
- * @param size
- * @return void*
+ * @param ptr   The pointer to realloc.
+ * @param size  The size of the memory to allocate.
+ * @param ...   The error handling code to execute if the allocation fails.
  */
-#ifndef CLI_REALLOC
-#define CLI_REALLOC(ptr, size, ...)          \
-    do {                                     \
-        void *vTmp = cli_realloc(ptr, size); \
-        if (NULL == vTmp) {                  \
-            do {                             \
-                __VA_ARGS__;                 \
-            } while (0);                     \
-            goto done;                       \
-        }                                    \
-        ptr = vTmp;                          \
+#ifndef CLI_MAX_REALLOC_OR_GOTO_DONE
+#define CLI_MAX_REALLOC_OR_GOTO_DONE(ptr, size, ...)     \
+    do {                                                 \
+        void *vTmp = cli_max_realloc((void *)ptr, size); \
+        if (NULL == vTmp) {                              \
+            do {                                         \
+                __VA_ARGS__;                             \
+            } while (0);                                 \
+            goto done;                                   \
+        }                                                \
+        ptr = vTmp;                                      \
     } while (0)
 #endif
 
-/*This is a duplicate from other PR's.*/
-#ifndef CLI_STRDUP
-#define CLI_STRDUP(buf, var, ...) \
-    do {                          \
-        var = cli_strdup(buf);    \
-        if (NULL == var) {        \
-            do {                  \
-                __VA_ARGS__;      \
-            } while (0);          \
-            goto done;            \
-        }                         \
+/**
+ * @brief Wrapper around realloc that, unlike some variants of realloc, will not free the ptr if size==0.
+ *
+ * IMPORTANT: This differs from realloc() in that if size==0, it will NOT free the ptr.
+ *
+ * NOTE: cli_safer_realloc() will NOT free ptr if size==0. It is safe to free ptr after `done:`.
+ *
+ * @param ptr   The pointer to realloc.
+ * @param size  The size of the memory to allocate.
+ * @param ...   The error handling code to execute if the allocation fails.
+ */
+#ifndef CLI_SAFER_REALLOC_OR_GOTO_DONE
+#define CLI_SAFER_REALLOC_OR_GOTO_DONE(ptr, size, ...)     \
+    do {                                                   \
+        void *vTmp = cli_safer_realloc((void *)ptr, size); \
+        if (NULL == vTmp) {                                \
+            do {                                           \
+                __VA_ARGS__;                               \
+            } while (0);                                   \
+            goto done;                                     \
+        }                                                  \
+        ptr = vTmp;                                        \
     } while (0)
 #endif
 
