@@ -1,7 +1,7 @@
 /*
  *  Phishing module: domain list implementation.
  *
- *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2024 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
@@ -25,10 +25,10 @@
 #define _PHISH_DOMAINCHECK_DB_H
 #include "clamav.h"
 
-int init_domainlist(struct cl_engine* engine);
-void domainlist_done(struct cl_engine* engine);
-void domainlist_cleanup(const struct cl_engine* engine);
-int is_domainlist_ok(const struct cl_engine* engine);
-int domainlist_match(const struct cl_engine* engine, char* real_url, const char* display_url, const struct pre_fixup_info* pre_fixup, int hostOnly);
+int init_domain_list(struct cl_engine* engine);
+void domain_list_done(struct cl_engine* engine);
+void domain_list_cleanup(const struct cl_engine* engine);
+int is_domain_list_ok(const struct cl_engine* engine);
+int domain_list_match(const struct cl_engine* engine, char* real_url, const char* display_url, const struct pre_fixup_info* pre_fixup, int hostOnly);
 
 #endif
